@@ -1,16 +1,17 @@
 """macro-kit is a package for macro recording and metaprogramming in Python."""
 
-__version__ = "0.3.8"
+__version__ = "0.3.9"
 __author__ = "Hanjin Liu"
 __email__ = "liuhanjin-sc@g.ecc.u-tokyo.ac.jp"
 
-from ._symbol import Symbol, register_type, COMPILED
-from .expression import Expr, Head, symbol
-from .macro import Macro, MacroFlags
-from .mock import Mock
-from .ast import parse
+from macrokit._symbol import Symbol, register_type, unregister_type, type_registered
+from macrokit.expression import Expr, Head, symbol, store
+from macrokit.macro import Macro, MacroFlags
+from macrokit.mock import Mock
+from macrokit.ast import parse
 from functools import wraps
 
+from macrokit._symbol import COMPILED
 
 __all__ = [
     "Symbol",
@@ -19,7 +20,10 @@ __all__ = [
     "Macro",
     "Mock",
     "symbol",
+    "store",
     "register_type",
+    "unregister_type",
+    "type_registered",
     "parse",
     "blocked",
     "record",
